
/* Chrysalide - Outil d'analyse de fichiers binaires
 * bits.h - équivalent Python du fichier "plugins/kaitai/parsers/bits.h"
 *
 * Copyright (C) 2019 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "bits.h"


#include <assert.h>
#include <pygobject.h>


#include <i18n.h>
#include <plugins/pychrysalide/access.h>
#include <plugins/pychrysalide/helpers.h>
#include <plugins/pychrysalide/analysis/content.h>
#include <plugins/pychrysalide/arch/vmpa.h>


#include "../record.h"
#include "../parsers/attribute.h"
#include "../../records/bits-int.h"



CREATE_DYN_CONSTRUCTOR(record_bit_field, G_TYPE_RECORD_BIT_FIELD);

/* Initialise une instance sur la base du dérivé de GObject. */
static int py_record_bit_field_init(PyObject *, PyObject *, PyObject *);

/* Lit la valeur d'un élément Kaitai entier représenté. */
static PyObject *py_record_bit_field_get_value(PyObject *, void *);



/******************************************************************************
*                                                                             *
*  Paramètres  : self = objet à initialiser (théoriquement).                  *
*                args = arguments fournis à l'appel.                          *
*                kwds = arguments de type key=val fournis.                    *
*                                                                             *
*  Description : Initialise une instance sur la base du dérivé de GObject.    *
*                                                                             *
*  Retour      : 0.                                                           *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static int py_record_bit_field_init(PyObject *self, PyObject *args, PyObject *kwds)
{
#if 0
    GKaitaiAttribute *attrib;               /* Attribut défini créateur    */
    GBinContent *content;                   /* Contenu binaire analysé     */
    mrange_t range;                         /* Espace couvert              */
    SourceEndian endian;                    /* Boutisme à observer         */
    int ret;                                /* Bilan de lecture des args.  */
    GRecordBitField *field;                 /* Création GLib à transmettre */
#endif

#define RECORD_BIT_FIELD_DOC                                                         \
    "The RecordItem class remembers a match between a described attribute and"  \
    " its concret value read from parsed binary data."                          \
    "\n"                                                                        \
    "Instances can be created using the following constructor:\n"               \
    "\n"                                                                        \
    "    RecordItem(content, range, endian, attrib)"                            \
    "\n"                                                                        \
    "Where the *attrib* arguments refers to a"                                  \
    " pychrysalide.plugins.kaitai.parsers.KaitaiAttribute instance as the"      \
    " creator of the newly created object, *content* points to a"               \
    " pychrysalide.analysis.BinContent instance, *range* is a"                  \
    " pychrysalide.arch.mrange object, *endian* states with a"                  \
    " pychrysalide.analysis.BinContent.SourceEndian hint the endianness used"   \
    " to read integer values."

    /* Récupération des paramètres */

#if 0   /* FIXME */

    ret = PyArg_ParseTuple(args, "O&O&O&",
                           convert_to_kaitai_attribute, &attrib,
                           convert_to_binary_content, &content,
                           convert_any_to_mrange, &range,
                           convert_to_binary_content, &endian);
    if (!ret) return -1;

    /* Initialisation d'un objet GLib */

    ret = forward_pygobjet_init(self);
    if (ret == -1) return -1;

    /* Eléments de base */

    field = G_RECORD_BIT_FIELD(pygobject_get(self));

    if (!g_record_bit_field_create(field, attrib, content, &range, endian))
    {
        PyErr_SetString(PyExc_ValueError, _("Unable to create record bit field."));
        return -1;
    }

    return 0;

#endif

    PyErr_SetString(PyExc_ValueError, _("Unable to create record bit field at the moment."));
    return -1;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : self    = objet Python concerné par l'appel.                 *
*                closure = non utilisé ici.                                   *
*                                                                             *
*  Description : Lit la valeur d'un élément Kaitai entier représenté.         *
*                                                                             *
*  Retour      : Bilan de l'opération.                                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static PyObject *py_record_bit_field_get_value(PyObject *self, void *closure)
{
    PyObject *result;                       /* Valeur à retourner          */
    GRecordBitField *field;                 /* Version native de l'élément */
    resolved_value_t resolved;              /* Valeur sous forme générique */
    bool status;                            /* Bilan d'opération           */

#define RECORD_BIT_FIELD_VALUE_ATTRIB PYTHON_GET_DEF_FULL   \
(                                                           \
    value, py_record_bit_field,                             \
    "Carried value (as integer), or None in case of error." \
)

    result = NULL;

    field = G_RECORD_BIT_FIELD(pygobject_get(self));

    status = g_record_bit_field_get_value(field, &resolved);

    if (status)
        switch (resolved.type)
        {
            case GVT_UNSIGNED_INTEGER:
                result = PyLong_FromUnsignedLongLong(resolved.unsigned_integer);
                break;

            default:
                assert(false);
                result = Py_None;
                Py_INCREF(result);
                break;

        }

    else
    {
        result = Py_None;
        Py_INCREF(result);
    }

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : -                                                            *
*                                                                             *
*  Description : Fournit un accès à une définition de type à diffuser.        *
*                                                                             *
*  Retour      : Définition d'objet pour Python.                              *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

PyTypeObject *get_python_record_bit_field_type(void)
{
    static PyMethodDef py_record_bit_field_methods[] = {
        { NULL }
    };

    static PyGetSetDef py_record_bit_field_getseters[] = {
        RECORD_BIT_FIELD_VALUE_ATTRIB,
        { NULL }
    };

    static PyTypeObject py_record_bit_field_type = {

        PyVarObject_HEAD_INIT(NULL, 0)

        .tp_name        = "pychrysalide.plugins.kaitai.records.RecordBitField",
        .tp_basicsize   = sizeof(PyGObject),

        .tp_flags       = Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,

        .tp_doc         = RECORD_BIT_FIELD_DOC,

        .tp_methods     = py_record_bit_field_methods,
        .tp_getset      = py_record_bit_field_getseters,

        .tp_init        = py_record_bit_field_init,
        .tp_new         = py_record_bit_field_new,

    };

    return &py_record_bit_field_type;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : -                                                            *
*                                                                             *
*  Description : Prend en charge l'objet 'pychrysalide.....RecordBitField.    *
*                                                                             *
*  Retour      : Bilan de l'opération.                                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool ensure_python_record_bit_field_is_registered(void)
{
    PyTypeObject *type;                     /* Type Python 'RecordItem'    */
    PyObject *module;                       /* Module à recompléter        */
    PyObject *dict;                         /* Dictionnaire du module      */

    type = get_python_record_bit_field_type();

    if (!PyType_HasFeature(type, Py_TPFLAGS_READY))
    {
        module = get_access_to_python_module("pychrysalide.plugins.kaitai.records");

        dict = PyModule_GetDict(module);

        if (!ensure_python_match_record_is_registered())
            return false;

        if (!register_class_for_pygobject(dict, G_TYPE_RECORD_BIT_FIELD, type))
            return false;

    }

    return true;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : arg = argument quelconque à tenter de convertir.             *
*                dst = destination des valeurs récupérées en cas de succès.   *
*                                                                             *
*  Description : Tente de convertir en correspondance attribut/binaire.       *
*                                                                             *
*  Retour      : Bilan de l'opération, voire indications supplémentaires.     *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

int convert_to_record_bit_field(PyObject *arg, void *dst)
{
    int result;                             /* Bilan à retourner           */

    result = PyObject_IsInstance(arg, (PyObject *)get_python_record_bit_field_type());

    switch (result)
    {
        case -1:
            /* L'exception est déjà fixée par Python */
            result = 0;
            break;

        case 0:
            PyErr_SetString(PyExc_TypeError, "unable to convert the provided argument to record bit field");
            break;

        case 1:
            *((GRecordBitField **)dst) = G_RECORD_BIT_FIELD(pygobject_get(arg));
            break;

        default:
            assert(false);
            break;

    }

    return result;

}
